"""
:authors: k0rsakov
:license: Apache License, Version 2.0, see LICENSE file
:copyright: (c) 2022 k0rsakov
"""


# __author__ = 'k0rsakov'
# __version__ = '1.0.0'
# __email__ = 'korsakov.iyu@gmail.com'